/* Copyright 2009 Fog Creek Software, Inc. */

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;

/* FogBugz namespaces-- make sure you add the neccesary assembly references to
 * DLL files contained in C:\Program Files\FogBugz\website\bin\ */
using FogCreek.FogBugz;
using FogCreek.FogBugz.Plugins;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz.UI;

namespace IPluginPageDisplay_Example
{
    /* Class Declaration: Inherit from Plugin, implement IPluginPageDisplay, etc. */
    public class IPluginPageDisplay_Example : Plugin, IPluginPageDisplay, IPluginExtrasMenu
    {
        /* Constructor: We'll just initialize the inherited Plugin class, which 
         * takes the passed instance of CPluginApi and sets its "api" member variable. */
        public IPluginPageDisplay_Example(CPluginApi api): base(api)
        {
        }

        #region IPluginPageDisplay Members

        public string PageDisplay()
        {
            string sHTML = "";
            sHTML += GetHeader();
            sHTML += GetForm();
            return sHTML;
        }
  
        public PermissionLevel PageVisibility()
        {
            /* We'll let all logged-in users with "normal" and "admin" permissions access
             * this page. */

            return PermissionLevel.Normal;
        }

        #endregion

        protected string GetHeader()
        {
            /* Generate an informational header, making use of the FogBugz UI library,
             * and grabbing this page's URL from the plugin API. */

            return String.Format(
                @"<p>{0}</p>
                  <p>
                      This plugin page, generated by the <b>IPluginPageDisplay</b>
                      interface, is not linked to automatically by FogBugz. Another
                      plugin interface must be used to add a link to this page
                      elsewhere on the FogBugz site. This particular plugin uses the
                      <b>IPluginExtrasMenu</b> interface to add a link to the ""Extras""
                      menu.
                  </p>               
                  <p>The Plugin API tells me this page's URL is: <b>{1}</b></p>
                  <p>&nbsp;</p>",

                FogCreek.FogBugz.UI.PageDisplay.Headline("Sample Plugin Page"),
                HttpUtility.HtmlEncode(api.Url.PluginPageUrl()));
        }

        protected string GetForm()
        {
            /* Generate a form that will post a value to the server and display
             * that value taken from the request object (if it exists).
             * 
             * IMPORTANT NOTE: The unique prefix of this plugin must be prepended to the
             * names of all form elements, but the values of those elements can then be
             * gathered from the request object without using the prefix.
             * 
             * In keeping with security best practices, this plugin does not display
             * unless a required action token validates.
             * 
             * In this case, the "color" value is posted and retrieved. */
            string sSelectedColor = "Blue";
            string sFavoriteColorResponse = "";
            if (api.Request[api.AddPluginPrefix("color")] != null)
            {
                if ((api.Request[api.AddPluginPrefix("actionToken")] != null) &&
                api.Security.ValidateActionToken(api.Request[api.AddPluginPrefix("actionToken")].ToString()))
                {
                    sSelectedColor = api.Request[api.AddPluginPrefix("color")].ToString();
                    sFavoriteColorResponse = GetFavoriteColorResponse();
                }
                else
                {
                    sSelectedColor = "Blue";
                    sFavoriteColorResponse = "Action token invalid!";
                }
            }

            return String.Format(
                @"<form action=""{0}"" method=""POST"">
                        <input type=""hidden"" name=""{1}actionToken"" value=""{2}"">
                        <p><b>Which color do you like best?</b></p>
                        <p>&nbsp;</p>
                        <div>
                             <b>I prefer:</b>&nbsp;&nbsp;{3}&nbsp;&nbsp;{4}&nbsp;&nbsp;
                             {5}
                        </div>
                        <div>&nbsp;</div>
                </form>",

                HttpUtility.HtmlAttributeEncode(api.Url.PluginPageUrl()),
                api.PluginPrefix,
                api.Security.GetActionToken(),
                Forms.SelectInput(api.PluginPrefix + "color",
                    new string[] { "Red", "Blue", "Green", "Orange" },
                    sSelectedColor),
                Forms.SubmitButton("idSubmit", "Submit"),
                sFavoriteColorResponse);
        }

        protected string GetFavoriteColorResponse()
        {
            /* No need to use the plugin prefix here. */

            if (api.Request[api.AddPluginPrefix("color")] == null)
                return "<span>&nbsp;</span>";
            else
                return String.Format(
                    @"<span style=""color:{0}""><b>Well, {0} is a nice
                    color.</b></span>",
                    HttpUtility.HtmlEncode(api.Request[api.AddPluginPrefix("color")].ToString()));
        }

        #region IPluginExtrasMenu Members

        public CNavMenuLink[] ExtrasMenuLinks()
        {
            /* Create a link to this plugin's page in the "Extras" menu */
            if (api.Person.GetCurrentPerson().GetPermissionLevel() > PermissionLevel.Community)
                return new CNavMenuLink[] { new CNavMenuLink("Tell us your favorite color",
                    api.Url.PluginPageUrl()) };
            else
                return new CNavMenuLink[] { };
        }

        #endregion
    }
}